# frozen_string_literal: true

require 'geokit'

DISTANCE_FOR_AVERAGING_IN_KILOMETERS = 0.33

class Track
  def initialize(track:)
    @track = track
  end

  def points
    @points ||= @track.points
  end

  def total_distance
    @track.distance
  end

  def total_elevation_gain
    sum = 0.0
    pair_size = points.size - 1

    pair_size.times do |i|
      sum += ascent(points[i], points[i + 1])
    end
    sum.floor
  end

  def ascent(start, finish)
    [0, finish.elevation - start.elevation].max
  end

  def distance_in_straigh_line(start: points.first, finish: points.last)
    Geokit::LatLng.new(*to_lat_lng(start))
                  .distance_to(to_lat_lng_str(finish)).round(3)
  end

  def pt_per_km
    (points.count / total_distance).round
  end

  def max_distance_from_the_start
    start = points.first
    points.map do |point|
      distance_in_straigh_line(start: start, finish: point)
    end.max
  end

  def back_to_start?
    distance_in_straigh_line(start: points.first, finish: points.last) < 0.3
  end

  def points_in_lan_lgt_format
    points.map { |p| [p.lat, p.lon] }
  end

  def averaged_points
    number_of_points = pt_per_km * DISTANCE_FOR_AVERAGING_IN_KILOMETERS
    points_in_lan_lgt_format.each_slice(number_of_points.ceil).map do |sector|
      sector.transpose.map { |ar| ar.sum(0.0) / ar.size }
    end
  end

  private

  def to_lat_lng(point)
    [point.lat, point.lon]
  end

  def to_lat_lng_str(point)
    "#{point.lat},#{point.lon}"
  end
end
